package de.mdelab.instanceGraphEditor.ui.actions;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import de.mdelab.instanceGraphEditor.ui.parts.NodesEditPart;


/**
 * An action that executes the command returned by the selected {@link EditPart} instances when given a  
 * {@link HideNodeAction#REQ_HIDE_NODE}.
 */
public class HideNodeAction extends SelectionAction {

    public static final String HIDE_NODE = "Hide_Node";
    public static final String REQ_HIDE_NODE = "Hide_Node";

    Request request;

    /**
     * Create a new instance of the class.
     * @param part
     */
    public HideNodeAction(IWorkbenchPart part) {
        super(part);
        setId(HIDE_NODE);
        setText("Hide Node");
        request = new Request(REQ_HIDE_NODE);
    }

    /**
     * Execute the commands that perform the {@link HideNodeAction#REQ_HIDE_NODE REQ_HIDE_NODE}.
     * 
     * It is assumed that this method is executed directly after
     * {@link HideNodeAction#calculateEnabled() calculateEnabled()}
     */
    @Override
    public void run() {
        // selected objects must be nodes because the action is enabled.
        @SuppressWarnings("unchecked") List<NodesEditPart> editParts = getSelectedObjects();
        CompoundCommand compoundCommand = new CompoundCommand();
		for (NodesEditPart nodeEditPart : editParts) {
			compoundCommand.add(nodeEditPart.getCommand(request));
		}
        execute(compoundCommand);
    }
    

    /**
     * {@inheritDoc}
     * <p>The action is enabled if all the selected entities on the
     * editor are {@link NodesEditPart} instances</p> 
     */
    @Override
    protected boolean calculateEnabled() {

    	if(getSelectedObjects().isEmpty()) {
            return true;
        }
        for(Object selectedObject : getSelectedObjects()) {
            if(!(selectedObject instanceof NodesEditPart)) {
                return false;
            }
        }
        return true;
    }
}
