package de.mdelab.instanceGraphEditor.ui.parts;

import java.util.EventObject;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.graph.DirectedGraph;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import de.mdelab.instanceGraphEditor.ui.model.Graph;
import de.mdelab.instanceGraphEditor.ui.policies.GraphXYLayoutEditPolicy;

public class GraphEditPart extends AbstractGraphicalEditPart {

	private GraphEditPartObserver observer;
	
	FreeformLayer layer = new FreeformLayer();

	public GraphEditPart(Graph graph) {
		setModel(graph);
		observer = new GraphEditPartObserver();
	}
	
	CommandStackListener stackListener = new CommandStackListener() {
		
		public void commandStackChanged(EventObject event) {
			if (!GraphAnimation.captureLayout(getFigure()))
				return;
			while (GraphAnimation.step())
				getFigure().getUpdateManager().performUpdate();
			GraphAnimation.end();
		}
	};

	protected List getModelChildren() {
		return ((Graph) getModel()).getNodes();
	}
	
	@Override
	public void activate() {
		super.activate();
		getViewer().getEditDomain().getCommandStack().addCommandStackListener(stackListener);
	}

	protected IFigure createFigure() {
		
		if(!((Graph)getModel()).isLayoutManualDesired()) 
			layer.setLayoutManager(new GraphLayoutManager(this));
		else
			layer.setLayoutManager(new FreeformLayout());
		layer.setBorder(new LineBorder(1));
		return layer;
	}
	
	public void checkLayout()
	{
		if(!((Graph)getModel()).isLayoutManualDesired()) {
			layer.setLayoutManager(new GraphLayoutManager(this));
		}
			
		else{
			layer.setLayoutManager(new FreeformLayout());
		}
	}
	
	public void contributeNodesToGraph(DirectedGraph graph, Map map) {
		
		GraphAnimation.recordInitialState(getContentPane());
		
		for (int i = 0; i < getChildren().size(); i++) {
			NodesEditPart node = (NodesEditPart) getChildren().get(i);
			node.contributeNodesToGraph(graph, map);
		}
	}
	
	public void contributeEdgesToGraph(DirectedGraph graph, Map map) {
		
		for (int i = 0; i < getChildren().size(); i++) {
			NodesEditPart child = (NodesEditPart) children.get(i);
			child.contributeEdgesToGraph(graph, map);
		}
	}
	
	protected void applyGraphResults(DirectedGraph graph, Map map) {
		applyChildrenResults(graph, map);
	}
	
	protected void applyChildrenResults(DirectedGraph graph, Map map) {
		for (int i = 0; i < getChildren().size(); i++) {
			NodesEditPart part = (NodesEditPart) getChildren().get(i);
			part.applyGraphResults(graph, map);
		}
	}

	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new GraphXYLayoutEditPolicy());
	}
	
	public class GraphEditPartObserver implements Observer {

		public GraphEditPartObserver() {
			Graph graph = (Graph) getModel();
			graph.addObserver(this);
		}

		@Override
		public void update(Observable arg0, Object arg1) {
			checkLayout();
			refresh();
		//	refreshChildren();
		//	refreshVisuals();
		//	refreshSourceConnections();
		//	refreshTargetConnections();
		}

	}
}
