package de.mdelab.sdm.interpreter.code.debug.ui.launcher;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;

public abstract class SDInterpreterLaunchMainTabDebug<Activity> extends SDInterpreterLaunchMainTab<Activity> {
	
	private Text	textServerAddress	= null;
	private Text	textServerPort		= null;

	private Button checkboxUseExternalInterpreter = null;

	@Override
	public void createControl( final Composite p_parent ) {
		super.createControl( p_parent );
		
		final Composite compositeMain =  (Composite) getControl();
		createDebugClientLaunchGroup( compositeMain );
	}

	private void createDebugClientLaunchGroup(final Composite p_compositeMain ) {
		final Group serverParametersGroup = createGroup( p_compositeMain, "Story Diagram Interpreter Debug Client", 1 );
		checkboxUseExternalInterpreter = createCheckButton( serverParametersGroup, "Use interpreter running on server:" ); 
		checkboxUseExternalInterpreter.addSelectionListener( new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent e)	{
				widgetSelected(e);
			}

			@Override
			public void widgetSelected(SelectionEvent e) {
				manageControlsEnablement();
				setDirty( true );
				
				updateLaunchConfigurationDialog();
			}
		} );

		createServerAddressGroup( serverParametersGroup );

		createServerPortGroup( serverParametersGroup );
	}
	
	private void manageControlsEnablement() {
		final boolean serverGroupEnabled = checkboxUseExternalInterpreter.getSelection();
		
		setLaunchGroupEnabled( !serverGroupEnabled );
		
		textServerAddress.setEnabled( serverGroupEnabled );
		textServerPort.setEnabled( serverGroupEnabled );
	}
	
	private void createServerAddressGroup( final Composite p_compositeMain ) {
		final Group groupServer = this.createGroup( p_compositeMain, "Server Adress", 1 );

		textServerAddress = createSingleText( groupServer, "Enter the adress of the server." );
		textServerAddress.addModifyListener( new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent e) {
				setDirty( true );
				updateLaunchConfigurationDialog();
			}
		} );
	}

	private void createServerPortGroup( Composite p_compositeMain )	{
		final Group groupServer = createGroup( p_compositeMain, "Server Port", 1 );

		textServerPort = createSingleText( groupServer, "Enter the server port." );
		textServerPort.addModifyListener( new ModifyListener() {

			@Override
			public void modifyText(ModifyEvent e) {
				setDirty( true );
				updateLaunchConfigurationDialog();
			}
		} );
	}

	@Override
	public boolean isValid( final ILaunchConfiguration p_configuration ) {
		try {
			if ( p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_USE_SERVER_INTERPRETER, false ) ) {
				setErrorMessage( null );

				final String serverAddressString = p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_SERVER_ADDRESS, "" );
					
				if ( serverAddressString.isEmpty() ) {
					setErrorMessage("Server address must be specified." );
					
					return false;
				}
		
				final String serverPortStr = p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_SERVER_PORT, "" );
				
				if ( serverPortStr.isEmpty() ) {
					setErrorMessage( "Server port must be specified." );
					
					return false;
				}
				
				try {
					final int serverPort = Integer.parseInt( serverPortStr  );

					if ( serverPort <= 0 ) {
						setErrorMessage( "Server port must be a strict positive number." );

						return false;
					}
				}
				catch ( final NumberFormatException p_ex ) {
					setErrorMessage( "Server port must be a number: " + p_ex.getLocalizedMessage() + "." );

					return false;
				}
				
				return true;
			}
			
			return super.isValid( p_configuration );
		}
		catch ( final CoreException p_ex ) {
			throw new RuntimeException( p_ex );
		}
	}

	@Override
	public void initializeFrom( final ILaunchConfiguration p_configuration ) {
		super.initializeFrom( p_configuration );

		initializeTextFieldFromAttribute( textServerAddress, ISDLaunchUIConstants.ATTR_SERVER_ADDRESS, p_configuration, "" );
		initializeTextFieldFromAttribute( textServerPort, ISDLaunchUIConstants.ATTR_SERVER_PORT, p_configuration, "" );
		
		try {
			checkboxUseExternalInterpreter.setSelection( p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_USE_SERVER_INTERPRETER, false ) );
			manageControlsEnablement();
		}
		catch ( final CoreException p_ex ) {
			p_ex.printStackTrace();
			
			throw new RuntimeException( p_ex );
		}
	}

	@Override
	public void performApply( final ILaunchConfigurationWorkingCopy p_configuration ) {
		super.performApply( p_configuration );

		p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_SERVER_ADDRESS, textServerAddress.getText() );
		p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_SERVER_PORT, textServerPort.getText() );
		p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_USE_SERVER_INTERPRETER, checkboxUseExternalInterpreter.getSelection() );
	}

	@Override
	public void setDefaults( final ILaunchConfigurationWorkingCopy p_configuration ) {
		super.setDefaults( p_configuration );

		p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_USE_SERVER_INTERPRETER, false );
		p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_SERVER_ADDRESS, ISDLaunchUIConstants.SERVER_DEFAULT_ADDRESS );
		p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_SERVER_PORT, ISDLaunchUIConstants.SERVER_DEFAULT_PORT );
	}
}
