
package de.mdelab.sdm.interpreter.code.debug.ui.launcher.table;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Class that plays the role of the domain model
 * 
 */
public class TableData<RootObject, RowObject> {

	private final List<RowObject> rowObjects;
	
	private final List<ITableViewerListener<RowObject>> listeners;
	
	/**
	 * Constructor
	 */
	public TableData() {
		super();
		
		rowObjects = new ArrayList<RowObject>();
		listeners = new ArrayList<ITableViewerListener<RowObject>>(); 
	}

	public void addTableViewerListener( final ITableViewerListener<RowObject> p_listener) {
		listeners.add( p_listener );
	}

	public boolean removeTableViewerListener( final ITableViewerListener<RowObject> p_listener) {
		return listeners.remove( p_listener );
	}
	
	public void clearRowObjects() {
		final Iterator<RowObject> rowsIt =  rowObjects.iterator();
		
		while ( rowsIt.hasNext() ) {
			final RowObject removedRow = rowsIt.next();
			rowsIt.remove();
			
			fireRowObjectRemoved( removedRow );
		}
	}
	
	public void addRowObject( final RowObject p_row ) {
		rowObjects.add( p_row );
		
		fireRowObjectAdded( p_row );
	}
	
	private void fireRowObjectAdded( final RowObject p_row ) {
		for ( final ITableViewerListener<RowObject> listener : listeners ) {
			listener.rowObjectAdded( p_row );
		}
	}
	
	public boolean removeRowObject( final RowObject p_row ) {
		final boolean removed = rowObjects.remove( p_row );
		
		fireRowObjectRemoved( p_row );
		
		return removed;
	}
	
	private void fireRowObjectRemoved( final RowObject p_row ) {
		for ( final ITableViewerListener<RowObject> listener : listeners ) {
			listener.rowObjectRemoved( p_row );
		}
	}

	/**
	 * Return the collection of activity parameters
	 */
	public List<RowObject> getRowObjects() {
		return rowObjects;
	}
}
