package de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.actparameter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ComboBoxViewerCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Table;

import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.ITableViewerListener;

public abstract class SDInstanceObjectEditingSupport<ActivityParameter extends SDTableActivityParameter<?, Classifier>, Classifier> extends EditingSupport {
	
	private final ILabelProvider labelProvider;
	
	private Set<Resource> instObjectsResources;
	
	private final List<ITableViewerListener<ActivityParameter>> editorListeners;

	public SDInstanceObjectEditingSupport( 	final TableViewer p_viewer,
											final ILabelProvider p_labelProvider ) {
	    super( p_viewer );
	    
	    labelProvider = p_labelProvider;
	    instObjectsResources = null;
	    editorListeners = new ArrayList<ITableViewerListener<ActivityParameter>>();
	}

	public void addEditorListener( final ITableViewerListener<ActivityParameter> p_listener ) {
		editorListeners.add( p_listener );
	}
	
	public boolean removeEditorListener( final ITableViewerListener<ActivityParameter> p_listener ) {
		return editorListeners.remove( p_listener );
	}
	
	protected abstract boolean isInstance( 	Classifier p_classifier,
											Object p_element );

	@Override
	protected CellEditor getCellEditor( final Object p_element ) {
	    @SuppressWarnings("unchecked")
		final ActivityParameter parameter = (ActivityParameter) p_element;
	   	final List<EObject> comboElementNames = new ArrayList<EObject>();

	   	for ( final Resource res : instObjectsResources ) {
		   	final Iterator<EObject> elementsIt = res.getAllContents();
		    while ( elementsIt.hasNext() ) {
		    	final EObject elem = elementsIt.next();
		    	  
		    	if ( isInstance( parameter.getType(), elem ) ) {
		    		comboElementNames.add( elem );
		    	}
		    }
	   	}
	      
	    Collections.sort( comboElementNames, new Comparator<EObject>() {

			@Override
			public int compare(	final EObject p_obj1,
								final EObject p_obj2 ) {
				return labelProvider.getText( p_obj1 ).compareTo( labelProvider.getText( p_obj2 ) );
			}
		} );
	    
		final ComboBoxViewerCellEditor editor = new ComboBoxViewerCellEditor( (Table) getViewer().getControl(), SWT.READ_ONLY );
		editor.setLabelProvider( labelProvider );
		editor.setContentProvider( ArrayContentProvider.getInstance() );
	    
		editor.setInput( comboElementNames );
 
	    return editor;
	}

	@Override
	protected boolean canEdit(  final Object p_element  ) {
		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	protected Object getValue(  final Object p_element  ) {
	    return ( (ActivityParameter) p_element ).getInstanceObject();
	}

	@Override
	protected void setValue( 	final Object p_element, 
			  					final Object p_userInputValue ) {
		@SuppressWarnings("unchecked")
		final ActivityParameter parameter = (ActivityParameter) p_element;
		parameter.setInstanceObject( (EObject) p_userInputValue );
	    
	    getViewer().update( p_element, null );
	    
	    fireParameterUpdated( parameter );
	}
	
	private void fireParameterUpdated( final ActivityParameter p_parameter ) {
	    for ( final ITableViewerListener<ActivityParameter> listener : editorListeners ) {
	    	listener.rowObjectUpdated( p_parameter );
	    }
	}

	public void setInstObjectsResources( final Set<Resource> p_instObjectsResources ) {
		instObjectsResources = p_instObjectsResources;
	}
}
